@@ -6,6 +6,7 @@ For development, see method `Features.get_evaluator` to confirm the
 naming specification.
 """
 
+from abc import ABC, abstractmethod
 from functools import cached_property
 from itertools import chain
 from os.path import basename
@@ -19,7 +20,7 @@ from matplotlib.axes import Axes
 from ..core import Population, Populations, Tree
 from ..utils import padding1d
 from .branch_features import BranchFeatures
-from .node_features import NodeFeatures
+from .node_features import BifurcationFeatures, NodeFeatures, TipFeatures
 from .path_features import PathFeatures
 from .sholl import Sholl
 
@@ -31,6 +32,12 @@ Feature = Literal[
     # node
     "node_radial_distance",
     "node_branch_order",
+    # bifurcation nodes
+    "bifurcation_radial_distance",
+    "bifurcation_branch_order",
+    # tip nodes
+    "tip_radial_distance",
+    "tip_branch_order",
     # branch
     "branch_length",
     "branch_tortuosity",
@@ -55,6 +62,10 @@ class Features:
     @cached_property
     def _node_features(self) -> NodeFeatures: return NodeFeatures(self.tree)
     @cached_property
+    def _bifurcation_features(self) -> BifurcationFeatures: return BifurcationFeatures(self._node_features)
+    @cached_property
+    def _tip_features(self) -> TipFeatures: return TipFeatures(self._node_features)
+    @cached_property
     def _branch_features(self) -> BranchFeatures: return BranchFeatures(self.tree)
     @cached_property
     def _path_features(self) -> PathFeatures: return PathFeatures(self.tree)
@@ -86,10 +97,12 @@ class Features:
         return np.array([self.tree.length(**kwargs)], dtype=np.float32)
 
     def get_sholl(self, **kwargs) -> npt.NDArray[np.float32]:
-        return Sholl(self.tree, **kwargs).get().astype(np.float32)
+        return Sholl(self.tree).get(**kwargs).astype(np.float32)
 
 
-class FeatureExtractor:
+class FeatureExtractor(ABC):
+    """Extract features from tree."""
+
     # fmt:off
     @overload
     def get(self, feature: Feature, **kwargs) -> npt.NDArray[np.float32]: ...
@@ -108,22 +121,29 @@ class FeatureExtractor:
         returns array of shape (N, L).
         """
         if isinstance(feature, dict):
-            return {k: self._get(k, **v) for k, v in feature.items()}
+            return {k: self._custom_get(k, **v) for k, v in feature.items()}
 
         if isinstance(feature, list):
-            return [self._get(k) for k in feature]
+            return [self._custom_get(k) for k in feature]
 
-        return self._get(feature, **kwargs)
+        return self._custom_get(feature, **kwargs)
 
     def plot(
         self, feature: FeatureWithKwargs, title: str | bool = True, **kwargs
-    ) -> Axes:  # TODO: sholl
+    ) -> Axes:
+        """Plot feature with appropriate way.
+
+        Notes
+        -----
+        The drawing method is different in different classes, different
+        in different features, and may different between versions,
+        there are NO guarantees.
+        """
         feat, _ = _get_feature_and_kwargs(feature)
         if not callable(plot := getattr(self, f"_plot_{feat}", None)):
             plot = self._plot  # default plot
 
-        vals = self._get(feature)
-        ax = plot(vals, **kwargs)
+        ax = plot(feature, **kwargs)
 
         if isinstance(title, str):
             ax.set_title(title)
@@ -132,12 +152,58 @@ class FeatureExtractor:
 
         return ax
 
+    def _custom_get(
+        self, feature: FeatureWithKwargs, **kwargs
+    ) -> npt.NDArray[np.float32]:
+        feat, _ = _get_feature_and_kwargs(feature)
+        if not callable(get := getattr(self, f"_get_{feat}", None)):
+            get = self._get  # default
+
+        return get(feature, **kwargs)
+
+    @abstractmethod
     def _get(self, feature: FeatureWithKwargs, **kwargs) -> npt.NDArray[np.float32]:
         raise NotImplementedError()
 
-    def _plot(self, vals: npt.NDArray[np.float32], **kwargs) -> Axes:
+    # pylint: disable=redefined-builtin
+    def _plot(
+        self, feature: FeatureWithKwargs, bins="auto", range=None, **kwargs
+    ) -> Axes:
+        vals = self._get(feature)
+        bin_edges = np.histogram_bin_edges(vals, bins, range)
+        return self._plot_histogram(vals, bin_edges, **kwargs)
+
+    @abstractmethod
+    def _plot_histogram(
+        self, vals: npt.NDArray[np.float32], bin_edges: npt.NDArray, **kwargs
+    ) -> Axes:
         raise NotImplementedError()
 
+    # custom features
+
+    def _plot_node_branch_order(self, *args, **kwargs) -> Axes:
+        return self._impl_plot_node_branch_order(*args, **kwargs)
+
+    def _plot_bifurcation_branch_order(self, *args, **kwargs) -> Axes:
+        return self._impl_plot_node_branch_order(*args, **kwargs)
+
+    def _plot_tip_branch_order(self, *args, **kwargs) -> Axes:
+        return self._impl_plot_node_branch_order(*args, **kwargs)
+
+    def _plot_sholl(self, feature: FeatureWithKwargs, **kwargs) -> Axes:
+        _, feat_kwargs = _get_feature_and_kwargs(feature)
+        step = feat_kwargs.get("step", 1)  # TODO: remove hard code
+        vals = self._get(feature)
+        bin_edges = np.arange(step / 2, step * (vals.shape[-1] + 1), step)
+        return self._plot_histogram(vals, bin_edges, **kwargs)
+
+    def _impl_plot_node_branch_order(
+        self, feature: FeatureWithKwargs, **kwargs
+    ) -> Axes:
+        vals = self._get(feature)
+        bin_edges = np.arange(int(np.ceil(vals.max() + 1)))
+        return self._plot_histogram(vals, bin_edges, **kwargs)
+
 
 class TreeFeatureExtractor(FeatureExtractor):
     """Extract feature from tree."""
@@ -153,20 +219,26 @@ class TreeFeatureExtractor(FeatureExtractor):
     def _get(self, feature: FeatureWithKwargs, **kwargs) -> npt.NDArray[np.float32]:
         return self._features.get(feature, **kwargs)
 
-    def _plot(self, vals: npt.NDArray[np.float32], **kwargs) -> Axes:
-        ax: Axes = sns.histplot(x=vals, **kwargs)
+    def _plot_histogram(
+        self, vals: npt.NDArray[np.float32], bin_edges: npt.NDArray, **kwargs
+    ) -> Axes:
+        weights = (vals != 0).astype(np.int32)
+        hist, _ = np.histogram(vals, bins=bin_edges, weights=weights)
+        x = (bin_edges[:-1] + bin_edges[1:]) / 2
+
+        ax: Axes = sns.barplot(x=x, y=hist, **kwargs)
         ax.set_ylabel("Count")
         return ax
 
-    def _plot_length(self, vals: npt.NDArray[np.float32], **kwargs) -> Axes:
+    def _plot_length(self, feature: FeatureWithKwargs, **kwargs) -> Axes:
         name = basename(self._tree.source)
-        ax: Axes = sns.barplot(x=[name], y=vals.squeeze(), **kwargs)
+        ax: Axes = sns.barplot(x=[name], y=self._get(feature).squeeze(), **kwargs)
         ax.set_ylabel("Length")
         return ax
 
 
 class PopulationFeatureExtractor(FeatureExtractor):
-    """Extract feature from population."""
+    """Extract features from population."""
 
     _population: Population
     _features: List[Features]
@@ -182,10 +254,12 @@ class PopulationFeatureExtractor(FeatureExtractor):
         v = np.stack([padding1d(len_max, v, dtype=np.float32) for v in vals])
         return v
 
-    def _plot(
-        self, vals: npt.NDArray[np.float32], bins="auto", range=None, **kwargs
+    def _get_sholl(self, feature: FeatureWithKwargs, **kwargs) -> npt.NDArray[np.float32]:
+        raise NotImplementedError()
+
+    def _plot_histogram(
+        self, vals: npt.NDArray[np.float32], bin_edges: npt.NDArray, **kwargs
     ) -> Axes:
-        bin_edges = np.histogram_bin_edges(vals, bins, range)
         hists = [
             np.histogram(v, bins=bin_edges, weights=(v != 0).astype(np.int32))[0]
             for v in vals
@@ -197,16 +271,18 @@ class PopulationFeatureExtractor(FeatureExtractor):
         ax.set_ylabel("Count")
         return ax
 
-    def _plot_length(self, vals: npt.NDArray[np.float32], **kwargs) -> Axes:
-        vals = vals.squeeze(axis=1)
+    def _plot_length(self, feature: FeatureWithKwargs, **kwargs) -> Axes:
         x = [basename(t.source) for t in self._population]
-        y = vals.flatten()
+        y = self._get(feature).flatten()
         ax: Axes = sns.barplot(x=x, y=y, **kwargs)
         ax.axhline(y=y.mean(), ls="--", lw=1)
         ax.set_ylabel("Length")
         ax.set_xticks([])
         return ax
 
+    def _plot_sholl(self, feature: FeatureWithKwargs, **kwargs) -> npt.NDArray[np.float32]:
+        raise NotImplementedError()
+
 
 class PopulationsFeatureExtractor(FeatureExtractor):
     """Extract feature from population."""
@@ -232,13 +308,12 @@ class PopulationsFeatureExtractor(FeatureExtractor):
 
         return out
 
-    def _plot(
-        self, vals: npt.NDArray[np.float32], bins="auto", range=None, **kwargs
+    def _plot_histogram(
+        self, vals: npt.NDArray[np.float32], bin_edges: npt.NDArray, **kwargs
     ) -> Axes:
-        bin_edges = np.histogram_bin_edges(vals, bins, range)
-        histogram = lambda v: np.histogram(
-            v, bins=bin_edges, weights=(v != 0).astype(np.int32)
-        )
+        def histogram(v):
+            return np.histogram(v, bins=bin_edges, weights=(v != 0).astype(np.int32))
+
         hists = [[histogram(t)[0] for t in p] for p in vals]
         hist = np.concatenate(hists).flatten()
 
@@ -253,8 +328,8 @@ class PopulationsFeatureExtractor(FeatureExtractor):
         ax.set_ylabel("Count")
         return ax
 
-    def _plot_length(self, vals: npt.NDArray[np.float32], **kwargs) -> Axes:
-        vals = vals.squeeze(axis=2)
+    def _plot_length(self, feature: FeatureWithKwargs, **kwargs) -> Axes:
+        vals = self._get(feature)
         labels = self._populations.labels
         x = np.concatenate([np.full(vals.shape[1], fill_value=i) for i in labels])
         y = vals.flatten()
@@ -279,5 +354,5 @@ def extract_feature(obj: Tree | Population) -> FeatureExtractor:
 def _get_feature_and_kwargs(feature: FeatureWithKwargs, **kwargs):
     if isinstance(feature, tuple):
         return feature[0], {**feature[1], **kwargs}
-    else:
-        return feature, kwargs
+
+    return feature, kwargs
