@@ -32,7 +32,7 @@ CameraPresets: Dict[CameraPreset, Camera] = {
 }
 CameraOptions = Vec3f | Tuple[Vec3f, Vec3f] | Tuple[Vec3f, Vec3f, Vec3f]
 Positions = Literal["lt", "lb", "rt", "rb"] | Tuple[float, float]
-positions = {
+locations = {
     "lt": (0.10, 0.90),
     "lb": (0.10, 0.10),
     "rt": (0.90, 0.90),
@@ -47,6 +47,7 @@ def draw(
     *,
     fig: Optional[Figure] = None,
     ax: Optional[Axes] = None,
+    show: bool | None = None,
     camera: CameraOptions | CameraPreset = "xy",
     color: Optional[Dict[int, str] | str] = None,
     label: str | bool = True,
@@ -62,10 +63,14 @@ def draw(
         If it is str, then it is treated as the path of swc file.
     fig : ~matplotlib.axes.Figure, optional
     ax : ~matplotlib.axes.Axes, optional
+    show : bool | None, default `None`
+        Wheather to call `plt.show()`. If not specified, it will depend
+        on if ax is passed in, it will not be called, otherwise it will
+        be called by default.
     camera : CameraOptions | CameraPreset, default "xy"
         Camera options (position, look-at, up). One, two, or three
         vectors are supported, if only one vector, then threat it as
-        look-at, so camera is ((0, 0, 0), look-at, (0, 1, 0));if two
+        look-at, so camera is ((0, 0, 0), look-at, (0, 1, 0)); if two
         vector, then then threat it as (look-at, up), so camera is
         ((0, 0, 0), look-at, up). An easy way is to use the presets
         "xy", "yz" and "zx".
@@ -77,7 +82,7 @@ def draw(
     direction_indicator : str or (float, float), default 'rb'
         Draw a xyz direction indicator, can be place on 'lt', 'lb',
         'rt', 'rb', or custom position.
-    unit : optional[str]
+    unit : str, optional
         Add unit text, e.g.: r"$\mu m$".
     **kwargs : dict[str, Unknown]
         Forwarded to `~matplotlib.collections.LineCollection`.
@@ -87,36 +92,41 @@ def draw(
     fig : ~matplotlib.axes.Figure
     ax : ~matplotlib.axes.Axes
     """
-
+    # pylint: disable=too-many-locals
     swc = Tree.from_swc(swc) if isinstance(swc, str) else swc
 
+    show = show is True or (show is None and ax is None)
     fig, ax = get_fig_ax(fig, ax)
+
     my_camera = _get_camera(camera)
     my_color = get_ax_color(ax, swc, color)
 
     xyz = swc.xyz()
     starts, ends = swc.id()[1:], swc.pid()[1:]
     lines = np.stack([xyz[starts], xyz[ends]], axis=1)
-    collection = draw_lines(lines, ax=ax, camera=my_camera, color=my_color, **kwargs)
+    collection = draw_lines(ax, lines, camera=my_camera, color=my_color, **kwargs)
 
     ax.autoscale()
-    _set_ax_memo(ax, swc, label=label, handle=collection)  # legend
+    _set_ax_memo(ax, swc, label=label, handle=collection)
 
     if len(get_ax_swc(ax)) == 1:
         ax.set_aspect(1)
         ax.spines[["top", "right"]].set_visible(False)
         if direction_indicator is not False:
-            p = (
-                positions[direction_indicator]
+            loc = (
+                locations[direction_indicator]
                 if isinstance(direction_indicator, str)
                 else direction_indicator
             )
-            draw_direction_indicator(ax=ax, camera=my_camera, position=p)
+            draw_direction_indicator(ax, camera=my_camera, loc=loc)
         if unit is not None:
             ax.text(0.05, 0.95, unit, transform=ax.transAxes)
     else:
         set_ax_legend(ax, loc="upper right")  # enable legend
 
+    if show:
+        fig.show()
+
     return fig, ax
 
 
@@ -151,14 +161,14 @@ def set_ax_legend(ax: Axes, *args, **kwargs) -> Legend | None:
     handles = ax_weak_memo[ax].get("handles", [])
 
     # filter `label = False`
-    handles = [a for i, a in enumerate(handles) if labels[i] != False]
-    labels = [a for i, a in enumerate(labels) if labels[i] != False]
+    handles = [a for i, a in enumerate(handles) if labels[i] is not False]
+    labels = [a for a in labels if a is not False]
 
-    if len(labels) > 0:
-        return ax.legend(handles, labels, *args, **kwargs)
-    else:
+    if len(labels) == 0:
         return None
 
+    return ax.legend(handles, labels, *args, **kwargs)
+
 
 def _set_ax_memo(
     ax: Axes,
