@@ -1,11 +1,11 @@
 """Neuron trees."""
 
+from . import swc_utils  # do not export all things from swc_utils
 from .branch import *
 from .branch_tree import *
 from .node import *
 from .path import *
 from .population import *
 from .swc import *
-from .swc_utils import *
 from .tree import *
 from .tree_utils import *
