@@ -1,22 +1,36 @@
 """SWC format."""
 
 import warnings
-from typing import Any, Dict, Iterable, List, Literal, Tuple, TypeVar, cast, overload
+from abc import ABC, abstractmethod
+from copy import deepcopy
+from typing import (
+    Any,
+    Dict,
+    Iterable,
+    List,
+    Literal,
+    Optional,
+    Tuple,
+    TypeVar,
+    cast,
+    overload,
+)
 
 import numpy as np
 import numpy.typing as npt
 import pandas as pd
 import scipy.sparse as sp
+from typing_extensions import Self
 
 from .swc_utils import (
-    check_single_root,
+    is_single_root,
     link_roots_to_nearest_,
     mark_roots_as_somas_,
     reset_index_,
     sort_nodes_,
 )
 
-__all__ = ["swc_cols", "eswc_cols", "read_swc", "SWCLike", "SWCTypeVar"]
+__all__ = ["swc_cols", "eswc_cols", "read_swc", "SWCLike", "DictSWC", "SWCTypeVar"]
 
 swc_cols: List[Tuple[str, npt.DTypeLike]] = [
     ("id", np.int32),
@@ -37,8 +51,8 @@ eswc_cols: List[Tuple[str, npt.DTypeLike]] = [
 ]
 
 
-class SWCLike:
-    """Abstract class that including swc infomation."""
+class SWCLike(ABC):
+    """ABC of SWC."""
 
     source: str = ""
 
@@ -77,13 +91,20 @@ class SWCLike:
         """Get the coordinates of shape(n_sample, 3)."""
         return np.stack([self.x(), self.y(), self.z()], axis=1)
 
+    def xyzw(self) -> npt.NDArray[np.float32]:
+        """Get the homogeneous coordinates of shape(n_sample, 4)."""
+        w = np.zeros_like(self.x())
+        return np.stack([self.x(), self.y(), self.z(), w], axis=1)
+
     def xyzr(self) -> npt.NDArray[np.float32]:
         """Get the coordinates and radius array of shape(n_sample, 4)."""
         return np.stack([self.x(), self.y(), self.z(), self.r()], axis=1)
 
+    @abstractmethod
     def keys(self) -> Iterable[str]:
         raise NotImplementedError()
 
+    @abstractmethod
     def get_ndata(self, key: str) -> npt.NDArray[Any]:
         raise NotImplementedError()
 
@@ -109,9 +130,9 @@ class SWCLike:
     # fmt: on
     def to_swc(
         self,
-        fname: str | None = None,
+        fname: Optional[str] = None,
         *,
-        extra_cols: List[str] | None = None,
+        extra_cols: Optional[List[str]] = None,
         source: bool = True,
         id_offset: int = 1,
     ) -> str | None:
@@ -123,6 +144,8 @@ class SWCLike:
         with open(fname, "w", encoding="utf-8") as f:
             f.writelines(it)
 
+        return None
+
     def _to_swc(
         self, extra_cols: List[str] | None, source: bool, id_offset: int
     ) -> Iterable[str]:
@@ -149,21 +172,37 @@ class SWCLike:
 
     # fmt: off
     @overload
-    def to_eswc(self, swc_path: str, **kwargs) -> None: ...
+    def to_eswc(self, fname: str, **kwargs) -> None: ...
     @overload
     def to_eswc(self, **kwargs) -> str: ...
     # fmt: on
-    def to_eswc(self, swc_path: str | None = None, **kwargs) -> str | None:
-        kwargs.setdefault("swc_path", swc_path)
-        kwargs.setdefault("extra_cols", [])
-        kwargs["extra_cols"].extend(k for k, t in eswc_cols)
-        return self.to_swc(**kwargs)
+    def to_eswc(
+        self,
+        fname: Optional[str] = None,
+        swc_path: Optional[str] = None,
+        extra_cols: Optional[List[str]] = None,
+        **kwargs,
+    ) -> str | None:
+        if swc_path is None:
+            warnings.warn(
+                "`swc_path` has been renamed to `fname` since v0.5.1, "
+                "and will be removed in next version",
+                DeprecationWarning,
+            )
+            fname = swc_path
+
+        extra_cols = extra_cols or []
+        extra_cols.extend(k for k, t in eswc_cols)
+        return self.to_swc(fname, extra_cols=extra_cols, **kwargs)  # type: ignore
 
 
 class DictSWC(SWCLike):
+    """SWC implementation on dict."""
+
     ndata: Dict[str, npt.NDArray]
 
     def __init__(self, **kwargs: npt.NDArray):
+        super().__init__()
         self.ndata = kwargs
 
     def keys(self) -> Iterable[str]:
@@ -172,6 +211,10 @@ class DictSWC(SWCLike):
     def get_ndata(self, key: str) -> npt.NDArray[Any]:
         return self.ndata[key]
 
+    def copy(self) -> Self:
+        """Make a copy."""
+        return deepcopy(self)
+
 
 SWCTypeVar = TypeVar("SWCTypeVar", bound=SWCLike)
 
@@ -230,7 +273,7 @@ def read_swc(
         reset_index_(df)
 
     # check swc
-    if not check_single_root(df):
+    if not is_single_root(df):
         warnings.warn(f"core: not signle root, swc: {swc_file}")
 
     if (df["pid"] == -1).argmax() != 0:
